/*==============================================================================
    FILE:           sys_services_uart.c

    DESCRIPTION:    Uses ADI System Services to interface with a SC58x UART.

    Copyright (c) 2019 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices SC58x family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#include "sys_services_uart.h"
#include <drivers/uart/adi_uart.h>
#include <services/pwr/adi_pwr.h>
#include <services/tmr/adi_tmr.h>
#include <services/gpio/adi_gpio.h>
#include <stddef.h>
#include <string.h>

#include <ADSP-SC58x_device.h>

#include "user_msd_host.h"

#define SYS_SERVICES_UART_NUM       0


/* UART driver memory */
uint8_t sys_services_uart_driver_memory[ADI_UART_BIDIR_INT_MEMORY_SIZE];
/* UART driver handle */
ADI_UART_HANDLE sys_services_uart_hdevice;

/* Set to true when the UART TX is idle (not transmitting). */
volatile bool sys_services_uart_tx_idle = true;

/* Serial data RX buffer.
   The Buffer size is set to 1 so the ADI_UART_EVENT_RX_BUFFER_PROCESSED event
   is called for every byte. */
#define SYS_SERVICES_UART_RX_BUFFER_SIZE        1
static unsigned char sys_services_uart_rx_buffer[SYS_SERVICES_UART_RX_BUFFER_SIZE];

static void sys_services_uart_irq_callback(void* pHandle, uint32_t u32Arg, void* pArg);

/*=============================================================================
Function:       sys_services_uart_init

Parameters:     None.

Description:    Initializes the ADI System Services UART interface..

Returns:        None.
==============================================================================*/
void sys_services_uart_init (void)
{
    ADI_UART_RESULT uart_result;

    uart_result = adi_uart_Open(SYS_SERVICES_UART_NUM, ADI_UART_DIR_BIDIRECTION, sys_services_uart_driver_memory, ADI_UART_BIDIR_INT_MEMORY_SIZE, &sys_services_uart_hdevice);
    if (uart_result == ADI_UART_SUCCESS)
    {
        uart_result = adi_uart_SetMode(sys_services_uart_hdevice, ADI_UART_MODE_UART);
        if (uart_result == ADI_UART_SUCCESS)
        {
            uart_result = adi_uart_EnableDMAMode(sys_services_uart_hdevice, false);
            if (uart_result == ADI_UART_SUCCESS)
            {
                /* register the callback function */
                uart_result = adi_uart_RegisterCallback(sys_services_uart_hdevice, sys_services_uart_irq_callback, (void*)0);
                if (uart_result == ADI_UART_SUCCESS)
                {
                    /* Disable the UART TX, RX and Line Status before re-configuring the UART. */
                    uart_result = adi_uart_EnableTx(sys_services_uart_hdevice, false);
                    uart_result = adi_uart_EnableRx(sys_services_uart_hdevice, false);
                    uart_result = adi_uart_EnableLineStatusEvents(sys_services_uart_hdevice, false);

                    uart_result =  adi_uart_SetBaudRate(sys_services_uart_hdevice, 115200);
                    if (uart_result == ADI_UART_SUCCESS)
                    {
                        uart_result = adi_uart_SetNumStopBits(sys_services_uart_hdevice, ADI_UART_ONE_STOPBIT);
                        if (uart_result == ADI_UART_SUCCESS)
                        {
                            uart_result = adi_uart_SetWordLen(sys_services_uart_hdevice, ADI_UART_WORDLEN_8BITS);
                            if (uart_result == ADI_UART_SUCCESS)
                            {
                                uart_result = adi_uart_SetParity(sys_services_uart_hdevice, ADI_UART_NO_PARITY);
                                if (uart_result == ADI_UART_SUCCESS)
                                {
                                    uart_result = adi_uart_SubmitRxBuffer(sys_services_uart_hdevice, sys_services_uart_rx_buffer, SYS_SERVICES_UART_RX_BUFFER_SIZE);

                                    //pADI_PORTC->MUX = (pADI_PORTC->MUX & ~((0x03 << 26)  (0x03 << 28)));
                                    pADI_PORTC->FER_SET = (1 << 13) | (1 << 14);

                                    /* Re-enable the UART RX and Line Status events. */
                                    uart_result = adi_uart_EnableRx(sys_services_uart_hdevice, true);
                                    uart_result = adi_uart_EnableLineStatusEvents(sys_services_uart_hdevice, true);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

/*=============================================================================
Function:       sys_services_uart_tx_buffer

Parameters:     length - number of bytes to transmit.
                p_buffer - pointer to the data buffer to transmit

Description:    Transmits the specified data to the ADI System Services controlled
                UART.

Returns:        CLD_SUCCESS - Data transmission started successfully.
                CLD_FAIL - Data transmission failed.
==============================================================================*/
CLD_RV sys_services_uart_tx_buffer (unsigned short length, unsigned char * p_buffer)
{
    if ((length <= ADI_UART_BIDIR_DMA_MEMORY_SIZE) && (sys_services_uart_tx_idle == true))
    {
        adi_uart_SubmitTxBuffer(sys_services_uart_hdevice, p_buffer, length);

        /* enable the UART transfer */
        adi_uart_EnableTx(sys_services_uart_hdevice, true);
        sys_services_uart_tx_idle = false;

        return CLD_SUCCESS;
    }
    else
    {
        return CLD_FAIL;
    }
}

/*=============================================================================
Function:       sys_services_uart_irq_callback

Parameters:     pHandle - ADI system services UART handle.
                u32Arg - The UART Event that occurred.
                pArg - Not used.

Description:    Function called by the ADI Systems Service UART driver when a
                UART interrupt occurred.

Returns:        None.
==============================================================================*/
static void sys_services_uart_irq_callback(void* pHandle, uint32_t u32Arg, void* pArg)
{
    ADI_UART_EVENT event    = (ADI_UART_EVENT)u32Arg;

    switch (event)
    {
        case ADI_UART_EVENT_TX_BUFFER_PROCESSED:
            sys_services_uart_tx_idle = true;

        break;
        case ADI_UART_EVENT_RX_BUFFER_PROCESSED:
            adi_uart_SubmitRxBuffer(sys_services_uart_hdevice, sys_services_uart_rx_buffer, SYS_SERVICES_UART_RX_BUFFER_SIZE);
            user_msd_host_rx_term_byte(sys_services_uart_rx_buffer[0]);
        break;

        case ADI_UART_EVENT_OVERRUN_ERROR:
        break;
        case ADI_UART_EVENT_PARITY_ERROR:
        break;
        case ADI_UART_EVENT_FRAMING_ERROR:
        break;
        case ADI_UART_EVENT_BREAK_INTERRUPT:
        break;
        case ADI_UART_EVENT_RX_FIFO_WM:
        case ADI_UART_EVENT_RECIEVED_ADDR:
        case ADI_UART_EVENT_AUTOBAUD_COMPLETE:
        case ADI_UART_EVENT_TX_COMPLETE:
        case ADI_UART_EVENT_RX_DMA_ERROR:
        case ADI_UART_EVENT_TX_DMA_ERROR:
        break;
        default:
        break;
    }
}

CLD_RV sys_services_uart_print (char * p_string)
{
    return sys_services_uart_tx_buffer(strlen(p_string), (unsigned char*)p_string);
}

CLD_Boolean sys_services_uart_idle (void)
{
    return (sys_services_uart_tx_idle == true ? CLD_TRUE : CLD_FALSE);
}
